/*
 * Teague Water Meter
 * Licensed under Creative Commons / GNU GPL where applicable.
 * Use it, make it better, and let others do the same!
 *
 * questions? akumpf@teauge.com
 */


//
// WiShield 1.0 -- MAKE SURE TO INLCUDE THE WISHIELD LIBRARY IN ARDUINO! :)
//
#include <WiShield.h>

#define WIRELESS_MODE_INFRA	1
#define WIRELESS_MODE_ADHOC	2

#define PIN_CONNECTION_LED 9

#include <avr/wdt.h>  	// Include this to handle watchdog timer 

// Wireless configuration parameters ----------------------------------------
unsigned char local_ip[] = {
  192,168,216,199};	// IP address of WiShield
unsigned char gateway_ip[] = {
  192,168,216,1};	// router or gateway IP address
unsigned char subnet_mask[] = {
  255,255,255,0};	// subnet mask for the local network
const prog_char ssid[] PROGMEM = {
  "Guests"};		// max 32 bytes

unsigned char security_type = 3;	// 0 - open; 1 - WEP; 2 - WPA; 3 - WPA2

// WPA/WPA2 passphrase
const prog_char security_passphrase[] PROGMEM = {
  "password_here"};	// max 64 characters

// WEP 128-bit keys
// sample HEX keys
prog_uchar wep_keys[] PROGMEM = {	
  0x6f, 0x75, 0x61, 0x69, 0x73, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Key 0
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Key 1
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Key 2
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00	// Key 3
};

// setup the wireless mode
// infrastructure - connect to AP
// adhoc - connect to another WiFi device
unsigned char wireless_mode = WIRELESS_MODE_INFRA;

unsigned char ssid_len;
unsigned char security_passphrase_len;
//---------------------------------------------------------------------------

inline void digitalInputWithPullup(byte pin, boolean b) {  
  pinMode(pin, INPUT);  
  digitalWrite(pin, b?HIGH:LOW); 
}

#define FLOW_PIN 3
#define FLOW_PIN_GND 5
#define TEMP_PIN 4

boolean firstSend = true;

void setup()
{
  Serial.begin(19200);
  Serial.println("Starting YellowJacket...");
  firstSend = true;
  digitalInputWithPullup(FLOW_PIN, true);
  pinMode(FLOW_PIN_GND, OUTPUT);
  digitalWrite(FLOW_PIN_GND, LOW);
  
  WiFi.init();
  Serial.println("Init complete!");
  //Serial.print("millis() = ");
  //Serial.println(millis());
}

// The stack does not have support for DNS and therefore cannot resolve
// host names. It needs actual IP addresses of the servers. This info
// can be obtained by executing, for example, $ ping twitter.com on
// a terminal on your PC
char teaguelabs_ip[] = {
  74,202,208,88};	// This points to Teague Labs -- MAKE THIS YOUR OWN SERVER'S IP ADDRESS :)

// This string can be used to send a request to Twitter.com to update your status
// It will need a valid Authorization string which can be derived from your
// Twitter.com username and password using Base64 algorithm
// See, http://en.wikipedia.org/wiki/Basic_access_authentication
// You need to replace <-!!-Authorization String-!!-> with a valid string before
// using this sample sketch.
// The Content-Length variable should equal the length of the data string
// In the example below, "Content-Length: 21" corresponds to "status=Ready to sleep"
//const prog_char twitter[] PROGMEM = {
//unsigned char twitter[] = {
//  "POST /statuses/update.xml HTTP/1.1\r\nAuthorization: Basic dGVhZ3VlbGFiczpwcm90MHR5cGU=\r\nUser-Agent: uIP/1.0\r\nHost: twitter.com\r\nContent-Length: 30\r\nContent-Type: application/x-www-form-urlencoded\r\n\r\nstatus=Sensing flow.0123456789"};
//extern char twitter2[];

int lastMinute = 0;
#define DATA_LEN 100 // 100
char dataStr[DATA_LEN];

const char USER_ID[] = "100000"; // Unique id for this node.
const char rootPageStr[] = "/Flow_Server_Code/addDataToFlow.php"; // MAKE THIS THE PATH TO YOUR SERVER DIRECTORY! :)

#define TEMP_MIN 350
#define TEMP_MAX 550

int ticksSinceLastSend = 1;
int latestTemp = 50;
boolean flowIsHigh = false;
int con = 1;

void loop()
{
  int f = digitalRead(FLOW_PIN);
  if(!flowIsHigh){
    if(f == 1){
      analogRead(TEMP_PIN); // dummy read to get ADC going (perhaps not necessary).
      latestTemp = (TEMP_MAX - min(TEMP_MAX, max(TEMP_MIN,  analogRead(TEMP_PIN))))/2;
      //latestTemp = analogRead(TEMP_PIN);
      ticksSinceLastSend++;
      delay(1); // to reduce bounce
      flowIsHigh = true;
    }
  }
  else{
    if(f == 0){
      flowIsHigh = false;
    }
  }
  // if this is the first iteration
  // send the request
  int minute = millis()/5000;
  if (minute != lastMinute) {
    if(ticksSinceLastSend != 0){
      if(firstSend){
        ticksSinceLastSend = 0;
        firstSend = false;
      }
      Serial.print("  Ticks: ");
      Serial.println(ticksSinceLastSend);
      Serial.print("  Con > ");
      sprintf(dataStr, "%s?key=YOURKEY&userID=%s&temp=%d&ticks=%d&secs=%d", rootPageStr, USER_ID, latestTemp, ticksSinceLastSend, 15);
      int ret = webclient_get(teaguelabs_ip, 80, dataStr);
      Serial.print("Resp=");
      Serial.println(ret);
      ticksSinceLastSend = 0;
    }
    lastMinute = minute;
    Serial.print("m=");
    Serial.println(lastMinute/4.0);    
    con = digitalRead(PIN_CONNECTION_LED);
    if(con != 1){
      // looks like we're not connected... wait a few sec and try again to make sure.
      delay(5000);
      con = digitalRead(PIN_CONNECTION_LED);
      if(con != 1){
        // really not connected!
        hardSafeReset();
      }
    }
  }
  WiFi.run();
}

void(* resetFunc) (void) = 0; //declare reset function @ address 0

void hardSafeReset(){
  Serial.println("CON RESTART");
  delay(50);
  resetFunc();  //call reset
  /*
  // HARD RESET..
  cli();			// Clear interrupts
  wdt_enable(WDTO_15MS);	// Set the Watchdog to 15ms
  while(1);		// Wait long enough that the reset should occur...
  // if it doesn't occur, try this.
  */
  WiFi.init();
  firstSend = true;
  ticksSinceLastSend = 1;
}
